import pymongo


class MongoDBHelper:
    """ MongoDB Helper"""

    def __init__(self, **kwargs):
        """
        Init Function
        :param kwargs:
        """
        self.client = pymongo.MongoClient(kwargs['uri'])
        self._database = getattr(self.client, kwargs['database'])

    def __enter__(self):
        """
        This function return self
        :return:
        """
        return self

    def __exit__(self, type, value, traceback):
        """
        This function connection close when context manager end
        :param kwargs:
        :return:
        """
        try:
            if self.client:
                self.client.close()
        except:
            pass

    @DeprecationWarning
    def insert(self, collection: str, data: dict) -> object:
        """
        This function insert data in collection
        :param collection:
        :param data:
        :return:object
        """
        return self._database[collection].insert(data)

    def insert_one(self, collection: str, data: dict) -> pymongo.InsertOne:
        """
        This function insert data in collection
        :param collection: str
        :param data: dict
        :return: pymongo.InsertOne
        """
        return self._database[collection].insert_one(data)

    def find_one(self, collection: str, query: dict, projection: dict = {}, default: object = None) -> dict:
        """
        This function get query result in collection
        :param collection: str
        :param query: dict
        :param projection: dict
        :return: dict
        """
        try:
            if bool(projection):
                return self._database[collection].find_one(query, projection)
            else:
                return self._database[collection].find_one(query)
        except Exception as e:
            if default:
                return default

            raise Exception(e)

    def find(self, collection: str, query: dict, projection: dict = {}, default: object = None) -> list:
        """
        This function list query results in collection
        :param collection: str
        :param query: dict
        :param projection: dict
        :param: default Object
        :return: list
        """
        try:
            if bool(projection):
                return self._database[collection].find(query, projection)
            else:
                return self._database[collection].find(query)
        except Exception as e:
            if default:
                return default

            raise Exception(e)

    def find_and_modify(self, collection: str, query: dict, **kwargs) -> list:
        """
        This function find and modify data in collection
        :param collection: 
        :param query: 
        :param kwargs: 
        :return: list
        """
        try:
            return self._database[collection].find_and_modify(
                query=query,
                update={"$set": kwargs},
                upsert=False,
                full_response=True
            )
        except Exception as e:
            raise Exception(e)

    def close(self):
        """
        This function call __exit__ function for close mongo connection
        """
        return self.__exit__



from enum import Enum

class HTTPMethods(Enum):
    delete = 0
    get = 1
    post = 2
    put = 3


class AuthenticationError(Exception):
    def __str__(self) -> str:
        return 'Your credentials invalid. Check your credentials.'


class SendProcessFailureError(Exception):
    def __str__(self) -> str:
        return 'Email sending procedure failed.'



""" Update Version Script """
with open("version.txt", "r") as version_file:
    version = version_file.read()
    version_numbers = version.split(".")

    version_numbers[-1] = str(int(version_numbers[-1]) + 1)
    new_version = ".".join(version_numbers)

with open("version.txt", "w") as version_file:
    version_file.write(new_version)


import requests
from utils import HTTPMethods, AuthenticationError, SendProcessFailureError
from statics import BASE_URL, ACCESS_URL_PATH, SMTP_URL_PATH

class SendPulse:
    """ SendPulse SMTP API """

    def __init__(self, client_id: str, client_secret: str):
        """
            :client_id:     string, SendPulse client id
            :client_secret: string, SendPulse client secret key
        """
        self._client_id = client_id
        self._client_secret = client_secret
        self._access_token = self.__get_access_token()


    def get_url(self, path: str=''):
        """
            Concanates path and url

            :path: string

            return concanated url
        """
        return BASE_URL + path


    def __get_header(self, use_access_token: bool=True):
        """
            Returns needed header

            return dict
        """
        return {'Authorization': 'Bearer ' + self._access_token} if use_access_token else {}


    def __perform_request(self, path: str, http_method: HTTPMethods, params: dict={}, body: dict={}, use_access_token: bool=True) -> dict:
        """
            Sends request and refreshes access_token if necessary 

            :path:             string, added to end of the base url
            :http_method:      int, enum from HTTPMethods
            :params:           dict, request parameters
            :json:             dict, request body
            :use_access_token: bool, whether header must include access_token
            :retry:            bool, if true retry on authentication failure 

            return response body from Sendpulse
        """
        request = getattr(requests, http_method.name)

        return request(
            url=self.get_url(path),
            headers=self.__get_header(use_access_token=use_access_token),
            params=params,
            json=body
        )

    
    def refresh_token(self):
        """
            Gets new access token
        """
        self._access_token = self.__get_access_token()


    def __get_access_token(self) -> str:
        """
            Gets access token

            :path: string, added to end of the base url

            return str, access token
        """
        response = self.__perform_request(path=ACCESS_URL_PATH, http_method=HTTPMethods.post, use_access_token=False, body={
            'grant_type': 'client_credentials',
            'client_id': self._client_id,
            'client_secret': self._client_secret,
        })

        if response.status_code == 200:
            return response.json()['access_token']
        else:
            raise AuthenticationError


    def send_email_with_template(self, subject: str, template_id: str, variables: dict, from_name: str, from_email: str, to_data: list):
        """
            Sends email by using templates on Sendpulse

            :subject:     string, email subject
            :template_id: string, template_id on SenpPulse
            :variables:   dict, {<variable_name>: <value>}
            :from_name:   string, email sender's name
            :from_email:  string, email sender's adress
            :to_data:     list, [{'name': <to_name>, 'email': <to_email>}]

            return response body from Sendpulse
        """
        try:
            return self.__perform_request(path=SMTP_URL_PATH, http_method=HTTPMethods.post, body={'email': {
                'subject': subject,
                'template': {
                    'id': template_id,
                    'variables': variables
                },
                'from': {
                    'name': from_name,
                    'email': from_email
                },
                'to': to_data
            }}).json()
        except:
            raise SendProcessFailureError


    def send_email(self, subject: str, html: str, text: str, from_name: str, from_email: str, to_data: list):
        """
            Sends email by using templates on Sendpulse

            :subject:     string, email subject
            :html:        string, html of email encoded in Base64
            :text:        string, text of email
            :from_name:   string, email sender's name
            :from_email:  string, email sender's adress
            :to_data:     list, [{'name': <to_name>, 'email': <to_email>}]

            return response body from Sendpulse
        """
        try:
            return self.__perform_request(path=SMTP_URL_PATH, http_method=HTTPMethods.post, body={'email': {
                'subject': subject,
                'html': html,
                'text': text,
                'from': {
                    'name': from_name,
                    'email': from_email
                },
                'to': to_data
            }}).json()
        except:
            raise SendProcessFailureError



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("version.txt", "r") as version_file:
    version = version_file.read()

setuptools.setup(
    name="heybooster-toolkit",
    version=version,
    author="Heybooster",
    author_email="hey@heybooster.ai",
    description="Heybooster Toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    url="https://github.com/hey-booster/heybooster-toolkit",
    project_urls={
        "Bug Tracker": "https://github.com/hey-booster/heybooster-toolkit/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
